#ifndef Knopf_H
#define Knopf_H

#include "TextFeld.h"

namespace Framework
{
    class TextFeld; // TextFeld.h
    class AlphaFeld; // AlphaFeld.h
    class Text; // Text.h
    class Schrift; // Schrift.h
    class LRahmen; // Rahmen.h
    class LTDBDatei; // Dateisystem.h
    class Knopf; // aus dieser Datei
    class KontrollKnopf; // aus dieser Datei

    // Eine Zeichnung des 2D GUI Frameworks, die einen Knopf zeichnet, den der Benutzer drcken kann
    class Knopf : public TextFeld
    {
    public:
        class Style : public ZeichnungHintergrund::Style
        {
        public:
            const static __int64 MehrfarbigText = 0x0010000; // Legt fest, ob der Text des Knopfes mehrere Farben haben darf
            const static __int64 KlickFarbe = 0x0020000; // Legt fest, ob der Knopf whrend er gedrckt wird eine Hintergrund Farbe hat
            const static __int64 KlickBild = 0x0040000; // Legt fest, ob der Knopf whrend er gedrckt wird ein Bild als Hintergrund hat
            const static __int64 KlickAlpha = 0x0080000; // Legt fest, ob der Knopf whrend er gedrckt wird fr das Zeichnen des Hintergrundes alpha blending verwendet
            const static __int64 KlickBuffer = 0x0100000; // Legt fest, ob der Knopf whrend er gedrckt wird einen Farbbergang besitzt

            const static __int64 Normal = Sichtbar | Erlaubt | Rahmen | Buffered | MehrfarbigText | KlickBuffer; // Vereint di Flags Sichtbar, Erlaubt, Rahmen, Buffered, MehrfarbigText, KlickBuffer
        };

    private:
        int klickFarbe;
        Bild *klickBild;
        AlphaFeld *klickBuffer;
        int klickIndex;
        int ref;

    public:
        // Konstruktor 
        __declspec( dllexport ) Knopf();
        // Destruktor 
        __declspec( dllexport ) virtual ~Knopf();
        // Setzt die Farbe, die als Hintergrund verwendet wird, wenn der Knopf gedrckt wird
        //  fc: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setKlickFarbe( int fc );
        // Setzt das Hintergrund Bild durch kopieren, das verwendet wird, wenn der Knopf gedrckt wird
        //  bild: Das Bild, das kopiert werden soll
        __declspec( dllexport ) void setKlickBild( Bild *bild );
        // Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird, wenn der Knopf gedrckt wird
        //  bild: Das neue Bild
        __declspec( dllexport ) void setKlickBildZ( Bild *bild );
        // Setzt einen Zeiger auf den Farbbergang, der verwendet wird, wenn der Knopf gedrckt wird
        //  af: Der neue Farbbergang
        __declspec( dllexport ) void setKBZ( AlphaFeld *af );
        // Setzt die Strke des Farbbergangs, der verwendet wird, wenn der Knopf gedrckt wird
        //  st: Die Strke
        __declspec( dllexport ) void setKBStrength( int st );
        // Setzt die Farbe des Farbbergangs, der verwendet wird, wenn der Knopf gedrckt wird
        //  fc: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setKBFarbe( int fc );
        // Verarbeitet Maus Nachrichten
        //  me: Das Ereignis, was durch die Mauseingabe ausgelt wurde
        __declspec( dllexport ) void doMausEreignis( MausEreignis &me ) override;
        // Zeichnet das Objekt nach zRObj, falls es sichtbar ist
        //  zRObj: Das Bild, in welches gezeichnet werden soll
        __declspec( dllexport ) void render( Bild &zRObj ) override;
        // Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) int getKlickFarbe() const;
        // Gibt das Hintergrund Bild zurck, das verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) Bild *getKlickBild() const;
        // Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck, das verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) Bild *zKlickBild() const;
        // Gibt den Farbbergang zurck, der verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) AlphaFeld *getKB() const;
        // Gibt den Farbbergang ohne erhhten Reference Counter zurck, der verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) AlphaFeld *zKB() const;
        // Gibt die Farbe des Farbbergangs im A8R8G8B8 Format zurck, der verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) int getKBFarbe() const;
        // Gibt die Strke des Farbbergangs zurck, der verwendet wird, wenn der Knopf gedrckt wird
        __declspec( dllexport ) int getKBStrength() const;
        // Kopiert den Knopf, so das er ohne auswirkungen auf das Original verndert werden kann
        __declspec( dllexport ) Zeichnung *dublizieren() const override;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) Knopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Objekt automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) Knopf *release();
    };

    // Eine Zeichnung des 2D GUI Frameworks, die einen Auswhlbares Kstchen mit beschriftung zeichnet
    class KontrollKnopf : public ZeichnungHintergrund
    {
    public:
        class Style : public ZeichnungHintergrund::Style
        {
        public:
            const static __int64 Selected = 0x00004; // Legt fest, ob das Kstchen ausgewhlt ist

            const static __int64 MehrfarbigText = 0x001000; // Legt Fest, ob der Beschriftungs Text mehrfarbig sein kann
            const static __int64 KlickFarbe = 0x002000; // Legt fest, ob das Kstchen whrend ed gedrckt wird eine Hintergrund Farbe hat
            const static __int64 KlickBild = 0x004000; // Legt fest, ob das Kstchen beim Klicken ein Hintergrund Bild hat
            const static __int64 KlickAlpha = 0x008000; // Legt fest, ob fr den Hintergrund des Kstchens beim Klicken alpha blending verwendet wird
            const static __int64 KlickBuffer = 0x010000; // Legt fest, ob das Kstchen beim Klicken einen Farbbergang hat
            const static __int64 SelectFarbe = 0x020000; // Legt fest, ob das Kstchen wrhend es ausgewhlt ist eine Hintergrund Farbe hat
            const static __int64 SelectBild = 0x040000; // Legt fest, ob das Kstchen wrhend es ausgewhlt ist ein Hintergrund Bild hat
            const static __int64 SelectAlpha = 0x080000; // Legt fest, ob das Kstchen wrhend es ausgewhlt ist fr den Hintergrund alpha blending verwendet
            const static __int64 SelectBuffer = 0x100000; // Legt fest, ob das Kstchen wrhend es ausgewhlt ist einen Farbbergang hat
            const static __int64 SelectText = 0x200000; // Legt fest, ob das Kstchen wrhend es ausgewhlt ist eine andere Beschriftung hat
            const static __int64 MehrzeiligText = 0x400000; // Legt fest, ob das Kstchen eine Mehrzeilige Beschriftung haben kann

            const static __int64 MausKlick = 0x800000; // Legt fest, ob gerade mit der Maus auf das Kstchen geklickt wird. (Wird vom Kstchen selbst verwaltet)

            const static __int64 Normal = Sichtbar | Erlaubt | Rahmen | KlickBuffer; // Vereint die Flags Sichtbar, Erlaubt, Rahmen, KlickBuffer
        };

    private:
        Text *txt;
        Text *sTxt;
        int sBgF;
        int kBgF;
        Bild *sBgB;
        Bild *kBgB;
        AlphaFeld *sAf;
        AlphaFeld *kAf;
        Bild *kasten;
        Bild *sKasten;
        Schrift *schrift;
        int sF;
        int sGr;
        int ref;

    public:
        // Konstruktor 
        __declspec( dllexport ) KontrollKnopf();
        // Destruktor 
        __declspec( dllexport ) virtual ~KontrollKnopf();
        // Setzt den Zeiger auf den Beschriftungs Text
        //  txt: Der Text
        __declspec( dllexport ) void setTextZ( Text *txt );
        // Setzt den Beschriftungs Text
        //  txt: Der Text
        __declspec( dllexport ) void setText( Text *txt );
        // Setzt den Beschriftungs Text
        //  txt: Der Text
        __declspec( dllexport ) void setText( const char *txt );
        // Setzt den Zeiger auf den Beschriftungs Text, der angezeigt wird, whrend das Kstchen ausgewhlt ist
        //  txt: Der Text
        __declspec( dllexport ) void setSTextZ( Text *txt );
        // Setzt den Beschriftungs Text, der angezeigt wird, whrend das Kstchen ausgewhlt ist
        //  txt: Der Text
        __declspec( dllexport ) void setSText( Text *txt );
        // Setzt den Beschriftungs Text, der angezeigt wird, whrend das Kstchen ausgewhlt ist
        //  txt: Der Text
        __declspec( dllexport ) void setSText( const char *txt );
        // Setzt die Schrift, die fr die Beschriftung verwendet werden soll
        //  schrift: Die Schrift
        __declspec( dllexport ) void setSchriftZ( Schrift *schrift );
        // Setzt die Farbe der Schrift, die fr die Beschriftung verwendet werden soll
        //  f: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setSFarbe( int f );
        // Setzt die Gre der Schrift, die fr die Beschriftung verwendet werden soll
        //  f: Die Hhe einer Zeile in Pixeln
        __declspec( dllexport ) void setSSize( int gr );
        // Setzt die Hintergrund Farbe, die verwendet wird, whrend das Kstchen ausgewhlt ist
        //  f: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setSBgFarbe( int f );
        // Setzt die Hintergrund Farbe, die verwendet wird, whrend auf das Kstchen geklickt wird
        //  f: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setKBgFarbe( int f );
        // Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird, whrend das Kstchen ausgewhlt ist
        //  b: Das Bild
        __declspec( dllexport ) void setSBgBildZ( Bild *b );
        // Setzt das Hintergrund Bild durch kopieren, das verwendet wird, whrend das Kstchen ausgewhlt ist
        //  b: Das Bild, das kopiert werden soll
        __declspec( dllexport ) void setSBgBild( Bild *b );
        // Setzt einen Zeiger auf das Hintergrund Bild, das verwendet wird, whrend auf das Kstchen geklickt wird
        //  b: Das Bild
        __declspec( dllexport ) void setKBgBildZ( Bild *b );
        // Setzt das Hintergrund Bild durch kopieren, das verwendet wird, whrend auf das Kstchen geklickt wird
        //  b: Das Bild, das kopiert werden soll
        __declspec( dllexport ) void setKBgBild( Bild *b );
        // Setzt einen Zeiger auf den Farbbergang, der verwendet wird, whrend das Kstchen ausgewhlt ist
        //  af: Der Farbbergang
        __declspec( dllexport ) void setSAlphaFeldZ( AlphaFeld *af );
        // Setzt die Farbe des Farbbergangs, der verwendet wird, whrend das Kstchen ausgewhlt ist
        //  f: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setSAFFarbe( int f );
        // Setzt die Strke des Farbbergangs, der verwendet wird, whrend das Kstchen ausgewhlt ist
        //  st: Die Strke
        __declspec( dllexport ) void setSAFStrength( int st );
        // Setzt einen Zeiger auf den Farbbergang, der verwendet wird, whrend auf das Kstchen geklickt wird
        //  af: Der Farbbergang
        __declspec( dllexport ) void setKAlphaFeldZ( AlphaFeld *af );
        // Setzt die Farbe des Farbbergangs, der verwendet wird, whrend auf das Kstchen geklickt wird
        //  f: Die Farbe im A8R8G8B8 Format
        __declspec( dllexport ) void setKAFFarbe( int f );
        // Setzt die Strke des Farbbergangs, der verwendet wird, whrend auf das Kstchen geklickt wird
        //  st: Die Strke
        __declspec( dllexport ) void setKAFStrength( int st );
        // Ldt die Bilder "kasten.gif" und "skasten.gif" aus einer LTDB Datei
        // Das Bild kasten.gif wird gezeichnet, wenn das Kstchen nicht ausgewhlt ist. Das Bild skasten.gif wird gezeichnet, wenn das Kstchen ausgewhlt wird
        //  zDat: Die LTDB Datei
        __declspec( dllexport ) void loadData( LTDBDatei *zDat );
        // Ldt die Bilder "kasten.gif" und "skasten.gif" aus einer LTDB Datei
        // Das Bild kasten.gif wird gezeichnet, wenn das Kstchen nicht ausgewhlt ist. Das Bild skasten.gif wird gezeichnet, wenn das Kstchen ausgewhlt wird
        //  ltdb: Der Pfad zur LTDB Datei
        __declspec( dllexport ) void loadData( const char *ltdb );
        // Verarbeitet Maus Nachrichten
        //  me: Das Ereignis, was durch die Mauseingabe ausgelt wurde
        __declspec( dllexport ) void doMausEreignis( MausEreignis &me ) override;
        // Zeichnet das Objekt nach zRObj, falls es sichtbar ist
        //  zRObj: Das Bild, in welches gezeichnet werden soll
        __declspec( dllexport ) void render( Bild &zRObj ) override;
        // Gibt die beschriftung zurck
        __declspec( dllexport ) Text *getText() const;
        // Gibt die beschriftung ohne erhhten Reference Counter zurck
        __declspec( dllexport ) Text *zText() const;
        // Gibt die beschriftung zurck, die verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) Text *getSText() const;
        // Gibt die beschriftung ohne erhhten Reference Counter zurck, die verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) Text *zSText() const;
        // Gibt die verwendete Schrift zurck
        __declspec( dllexport ) Schrift *getSchrift() const;
        // Gibt die verwendete Schrift ohne Reference Counter zurck
        __declspec( dllexport ) Schrift *zSchrift() const;
        // Gibt die Schrift Farbe im A8R8G8B8 Format zurck
        __declspec( dllexport ) int getSFarbe() const;
        // Gibt die Schrift Gre zurck
        __declspec( dllexport ) int getSSize() const;
        // Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) int getSBgFarbe() const;
        // Gibt die Hintergrund Farbe im A8R8G8B8 Format zurck, die verwendet wird, whrend auf das Kstchen geklickt wird
        __declspec( dllexport ) int getKBgFarbe() const;
        // Gibt das Hintergrund Bild zurck, das verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) Bild *getSBgBild() const;
        // Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck, das verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) Bild *zSBgBild() const;
        // Gibt das Hintergrund Bild zurck, das verwendet wird, whrend auf das Kstchen geklickt wird
        __declspec( dllexport ) Bild *getKBgBild() const;
        // Gibt das Hintergrund Bild ohne erhhten Reference Counter zurck, das verwendet wird, whrend auf das Kstchen geklickt wird
        __declspec( dllexport ) Bild *zKBgBild() const;
        // Gibt den Farbbergang zurck, der verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) AlphaFeld *getSAlphaFeld() const;
        // Gibt den Farbbergang ohne erhhten Reference COunter zurck, der verwendet wird, whrend das Kstchen ausgewhlt ist
        __declspec( dllexport ) AlphaFeld *zSAlphaFeld() const;
        // Gibt den Farbbergang zurck, der verwendet wird, whrend auf das Kstchen geklickt wird
        __declspec( dllexport ) AlphaFeld *getKAlphaFeld() const;
        // Gibt den Farbbergang ohne erhhten Reference COunter zurck, der verwendet wird, whrend auf das Kstchen geklickt wird
        __declspec( dllexport ) AlphaFeld *zKAlphaFeld() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) KontrollKnopf *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Objekt automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) KontrollKnopf *release();
    };
}

#endif